from pathlib import Path
import os
import shutil
from dataclasses import dataclass

from markdown_it import MarkdownIt


@dataclass(frozen=True)
class Config:
    source_dir: Path
    build_dir: Path


def _read_if_exists(path: Path) -> str:
    if path.exists():
        with open(path, "r", encoding="utf-8") as f:
            return f.read()
    return ""


def _replace_content(src: Path, dst: Path):
    if src.exists():
        if dst.exists():
            shutil.rmtree(dst)
        shutil.copytree(src, dst)


def _write_file(path: Path, content: str):
    os.makedirs(path.parent, exist_ok=True)
    print("writing to:", path)
    with open(path, "w", encoding="utf-8") as f:
        f.write(content)


def _read_file(path: Path) -> str:
    with open(path, "r", encoding="utf-8") as f:
        return f.read()


def _get_transformed_path(config: Config, path: Path, ext: str) -> str:
    source_dir = path.parent.relative_to(config.source_dir)
    filename = str(path.stem) + f".{ext}"
    return config.build_dir / source_dir / filename


def generate(config: Config):

    _replace_content(config.source_dir / "static", config.build_dir / "static")

    template_dir = config.source_dir / "templates"
    header = _read_if_exists(template_dir / "header.html")
    footer = _read_if_exists(template_dir / "footer.html")

    parser = MarkdownIt()
    sources = config.source_dir.glob("**/*.md")
    for source in sources:
        print("processing source: ", source)
        content = _read_file(source)
        output = parser.render(content)
        output_path = _get_transformed_path(config, source, "html")
        _write_file(output_path, header + output + footer)
